#!/usr/bin/env node
var path = require('path');
var arg = require('arg');
var fsp = require('fs/promises');
var require$$0 = require('tty');
var bunchee = require('bunchee');
var fs = require('fs');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var path__default = /*#__PURE__*/_interopDefault(path);
var arg__default = /*#__PURE__*/_interopDefault(arg);
var fsp__default = /*#__PURE__*/_interopDefault(fsp);
var require$$0__default = /*#__PURE__*/_interopDefault(require$$0);
var fs__default = /*#__PURE__*/_interopDefault(fs);

const availableExtensions = [
    'js',
    'cjs',
    'mjs',
    'jsx',
    'ts',
    'tsx',
    'cts',
    'mts'
];
const SRC = 'src';
const dtsExtensionsMap = {
    js: 'd.ts',
    cjs: 'd.cts',
    mjs: 'd.mts'
};
const tsExtensions = [
    'ts',
    'tsx',
    'cts',
    'mts'
];

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

var picocolors = {exports: {}};

let tty = require$$0__default.default;
let isColorSupported = !("NO_COLOR" in process.env || process.argv.includes("--no-color")) && ("FORCE_COLOR" in process.env || process.argv.includes("--color") || process.platform === "win32" || tty.isatty(1) && process.env.TERM !== "dumb" || "CI" in process.env);
let formatter = (open, close, replace = open)=>(input)=>{
        let string = "" + input;
        let index = string.indexOf(close, open.length);
        return ~index ? open + replaceClose(string, close, replace, index) + close : open + string + close;
    };
let replaceClose = (string, close, replace, index)=>{
    let start = string.substring(0, index) + replace;
    let end = string.substring(index + close.length);
    let nextIndex = end.indexOf(close);
    return ~nextIndex ? start + replaceClose(end, close, replace, nextIndex) : start + end;
};
let createColors = (enabled = isColorSupported)=>({
        isColorSupported: enabled,
        reset: enabled ? (s)=>`\x1b[0m${s}\x1b[0m` : String,
        bold: enabled ? formatter("\x1b[1m", "\x1b[22m", "\x1b[22m\x1b[1m") : String,
        dim: enabled ? formatter("\x1b[2m", "\x1b[22m", "\x1b[22m\x1b[2m") : String,
        italic: enabled ? formatter("\x1b[3m", "\x1b[23m") : String,
        underline: enabled ? formatter("\x1b[4m", "\x1b[24m") : String,
        inverse: enabled ? formatter("\x1b[7m", "\x1b[27m") : String,
        hidden: enabled ? formatter("\x1b[8m", "\x1b[28m") : String,
        strikethrough: enabled ? formatter("\x1b[9m", "\x1b[29m") : String,
        black: enabled ? formatter("\x1b[30m", "\x1b[39m") : String,
        red: enabled ? formatter("\x1b[31m", "\x1b[39m") : String,
        green: enabled ? formatter("\x1b[32m", "\x1b[39m") : String,
        yellow: enabled ? formatter("\x1b[33m", "\x1b[39m") : String,
        blue: enabled ? formatter("\x1b[34m", "\x1b[39m") : String,
        magenta: enabled ? formatter("\x1b[35m", "\x1b[39m") : String,
        cyan: enabled ? formatter("\x1b[36m", "\x1b[39m") : String,
        white: enabled ? formatter("\x1b[37m", "\x1b[39m") : String,
        gray: enabled ? formatter("\x1b[90m", "\x1b[39m") : String,
        bgBlack: enabled ? formatter("\x1b[40m", "\x1b[49m") : String,
        bgRed: enabled ? formatter("\x1b[41m", "\x1b[49m") : String,
        bgGreen: enabled ? formatter("\x1b[42m", "\x1b[49m") : String,
        bgYellow: enabled ? formatter("\x1b[43m", "\x1b[49m") : String,
        bgBlue: enabled ? formatter("\x1b[44m", "\x1b[49m") : String,
        bgMagenta: enabled ? formatter("\x1b[45m", "\x1b[49m") : String,
        bgCyan: enabled ? formatter("\x1b[46m", "\x1b[49m") : String,
        bgWhite: enabled ? formatter("\x1b[47m", "\x1b[49m") : String
    });
picocolors.exports = createColors();
picocolors.exports.createColors = createColors;
var picocolorsExports = picocolors.exports;
var pc = /*@__PURE__*/ getDefaultExportFromCjs(picocolorsExports);

const defaultColorFn = (text)=>text;
function color(prefixColor) {
    return pc.isColorSupported ? pc[prefixColor] : defaultColorFn;
}
const logger = {
    log (...arg) {
        console.log(...arg);
    },
    warn (...arg) {
        console.warn(color('yellow')('⚠️'), ...arg);
    },
    error (...arg) {
        console.error(color('red')('⨯'), ...arg);
    },
    info (...arg) {
        console.log(color('green')('✓'), ...arg);
    }
};
function paint(prefix, prefixColor, ...arg) {
    if (pc.isColorSupported) {
        console.log(pc[prefixColor](prefix), ...arg);
    } else {
        console.log(prefix, ...arg);
    }
}

function exit(err) {
    logger.error(err);
    process.exit(1);
}
async function hasPackageJson(cwd) {
    return await fileExists(path__default.default.resolve(cwd, 'package.json'));
}
async function getPackageMeta(cwd) {
    const pkgFilePath = path__default.default.resolve(cwd, 'package.json');
    let targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(await fsp__default.default.readFile(pkgFilePath, {
            encoding: 'utf-8'
        }));
    } catch (_) {}
    return targetPackageJson;
}
function isTypescriptFile(filename) {
    const ext = path__default.default.extname(filename).slice(1);
    return tsExtensions.includes(ext);
}
async function fileExists(filePath) {
    try {
        await fsp__default.default.access(filePath);
        return true;
    } catch (err) {
        if (err.code === 'ENOENT') {
            return false;
        }
        throw err;
    }
}
const hasAvailableExtension = (filename)=>availableExtensions.includes(path__default.default.extname(filename).slice(1));
const baseNameWithoutExtension = (filename)=>path__default.default.basename(filename, path__default.default.extname(filename));

var version = "4.3.0";

function relativify(path) {
    return path.startsWith('.') ? path : `./${path}`;
}

const DIST = 'dist';
// Output with posix style in package.json
function getDistPath(...subPaths) {
    return `./${DIST}/${subPaths.join('/')}`;
}
const normalizeBaseNameToExportName = (baseName)=>{
    return /^index(\.|$)/.test(baseName) ? '.' : relativify(baseName);
};
function createExportCondition(exportName, sourceFile, moduleType) {
    const isTsSourceFile = isTypescriptFile(sourceFile);
    let cjsExtension = 'js';
    if (moduleType === 'module') {
        cjsExtension = 'cjs';
    }
    if (isTsSourceFile) {
        return {
            import: {
                types: getDistPath('es', `${exportName}.d.mts`),
                default: getDistPath('es', `${exportName}.mjs`)
            },
            require: {
                types: getDistPath('cjs', `${exportName}.${dtsExtensionsMap[cjsExtension]}`),
                default: getDistPath('cjs', `${exportName}.${cjsExtension}`)
            }
        };
    }
    return {
        import: getDistPath(`${exportName}.mjs`),
        require: getDistPath(`${exportName}.${cjsExtension}`)
    };
}
async function collectSourceEntries(sourceFolderPath) {
    const bins = new Map();
    const exportsEntries = new Map();
    const entryFileDirentList = await fsp__default.default.readdir(sourceFolderPath, {
        withFileTypes: true
    });
    for (const dirent of entryFileDirentList){
        if (dirent.isDirectory()) {
            if (dirent.name === 'bin') {
                const binDirentList = await fsp__default.default.readdir(path__default.default.join(sourceFolderPath, dirent.name), {
                    withFileTypes: true
                });
                for (const binDirent of binDirentList){
                    if (binDirent.isFile()) {
                        const binFile = path__default.default.join(sourceFolderPath, dirent.name, binDirent.name);
                        const binName = baseNameWithoutExtension(binDirent.name);
                        if (fs__default.default.existsSync(binFile)) {
                            bins.set(binName, binDirent.name);
                        }
                    }
                }
            } else {
                // Search folder/<index>.<ext> convention entries
                const extensions = availableExtensions;
                for (const extension of extensions){
                    const indexFile = path__default.default.join(sourceFolderPath, dirent.name, `index.${extension}`);
                    if (fs__default.default.existsSync(indexFile)) {
                        exportsEntries.set(dirent.name, indexFile);
                        break;
                    }
                }
            }
        } else if (dirent.isFile()) {
            const isAvailableExtension = availableExtensions.includes(path__default.default.extname(dirent.name).slice(1));
            if (isAvailableExtension) {
                const baseName = baseNameWithoutExtension(dirent.name);
                const isBinFile = baseName === 'bin';
                if (isBinFile) {
                    bins.set('.', dirent.name);
                } else {
                    if (hasAvailableExtension(dirent.name)) {
                        exportsEntries.set(baseName, dirent.name);
                    }
                }
            }
        }
    }
    return {
        bins,
        exportsEntries
    };
}
function createExportConditionPair(exportName, sourceFile, moduleType) {
    // <exportName>.<specialCondition>
    let specialCondition;
    let exportCondName;
    if (exportName.indexOf('.') > 0) {
        const [originExportName, specialConditionName] = exportName.split('.');
        specialCondition = {
            [specialConditionName]: getDistPath('es', `${originExportName}-${specialConditionName}.mjs`)
        };
        exportCondName = normalizeBaseNameToExportName(originExportName);
        return [
            exportCondName,
            specialCondition
        ];
    }
    exportCondName = normalizeBaseNameToExportName(exportName);
    const exportCond = createExportCondition(exportName, sourceFile, moduleType);
    return [
        exportCondName,
        exportCond
    ];
}
async function prepare(cwd) {
    const sourceFolder = path__default.default.resolve(cwd, SRC);
    if (!fs__default.default.existsSync(sourceFolder)) {
        logger.error(`Source folder ${sourceFolder} does not exist. Cannot proceed to configure \`exports\` field.`);
        process.exit(1);
    }
    const pkgJsonPath = path__default.default.join(cwd, 'package.json');
    let pkgJson = {};
    if (fs__default.default.existsSync(pkgJsonPath)) {
        const pkgJsonString = await fsp__default.default.readFile(pkgJsonPath, 'utf-8');
        pkgJson = JSON.parse(pkgJsonString);
    }
    // configure `files` field with `dist`
    const files = pkgJson.files || [];
    if (!files.includes(DIST)) {
        files.push(DIST);
    }
    pkgJson.files = files;
    let isUsingTs = false;
    // Collect bins and exports entries
    const { bins, exportsEntries } = await collectSourceEntries(sourceFolder);
    const tsconfigPath = path__default.default.join(cwd, 'tsconfig.json');
    if (!fs__default.default.existsSync(tsconfigPath)) {
        const sourceFiles = [
            ...exportsEntries.values()
        ].concat([
            ...bins.values()
        ]);
        const hasTypeScriptFiles = sourceFiles.some((filename)=>isTypescriptFile(filename));
        if (hasTypeScriptFiles) {
            isUsingTs = true;
            await fsp__default.default.writeFile(tsconfigPath, '{}', 'utf-8');
            logger.log(`Detected using TypeScript but tsconfig.json is missing, created a ${pc.blue('tsconfig.json')} for you.`);
        }
    }
    // Configure as ESM package by default if there's no `type` field
    if (!pkgJson.type) {
        pkgJson.type = 'module';
    }
    if (bins.size > 0) {
        logger.log('Discovered binaries entries:');
        const maxLengthOfBinName = Math.max(...Array.from(bins.keys()).map((binName)=>normalizeBaseNameToExportName(binName).length));
        for (const [binName, binFile] of bins.entries()){
            const spaces = ' '.repeat(Math.max(maxLengthOfBinName - normalizeBaseNameToExportName(binName).length, 0));
            logger.log(`  ${normalizeBaseNameToExportName(binName)}${spaces}: ${binFile}`);
        }
        if (bins.size === 1 && bins.has('.')) {
            pkgJson.bin = getDistPath('bin', 'index.js');
        } else {
            pkgJson.bin = {};
            for (const [binName] of bins.entries()){
                pkgJson.bin[binName === '.' ? pkgJson.name : binName] = getDistPath('bin', binName + '.js');
            }
        }
    }
    if (exportsEntries.size > 0) {
        logger.log('Discovered exports entries:');
        const maxLengthOfExportName = Math.max(...Array.from(exportsEntries.keys()).map((exportName)=>normalizeBaseNameToExportName(exportName).length));
        for (const [exportName, exportFile] of exportsEntries.entries()){
            const spaces = ' '.repeat(Math.max(maxLengthOfExportName - normalizeBaseNameToExportName(exportName).length, 0));
            logger.log(`  ${normalizeBaseNameToExportName(exportName)}${spaces}: ${exportFile}`);
        }
        const pkgExports = {};
        for (const [exportName, sourceFile] of exportsEntries.entries()){
            const [key, value] = createExportConditionPair(exportName, sourceFile, pkgJson.type);
            pkgExports[key] = {
                ...value,
                ...pkgExports[key]
            };
        }
        // Configure node10 module resolution
        if (exportsEntries.has('index')) {
            const isESM = pkgJson.type === 'module';
            const mainExport = pkgExports['.'];
            const mainCondition = isESM ? 'import' : 'require';
            pkgJson.main = isUsingTs ? mainExport[mainCondition].default : mainExport[mainCondition];
            pkgJson.module = isUsingTs ? mainExport.import.default : mainExport.import;
            if (isUsingTs) {
                pkgJson.types = isESM ? mainExport.import.types : mainExport.require.types;
            }
        }
        // Assign the properties by order: files, main, module, types, exports
        if (Object.keys(pkgExports).length > 0) {
            pkgJson.exports = pkgExports;
        }
    }
    await fsp__default.default.writeFile(pkgJsonPath, JSON.stringify(pkgJson, null, 2));
    logger.info('Configured `exports` in package.json');
}

const helpMessage = `
Usage: bunchee [options]

Options:
  -v, --version          output the version number
  -w, --watch            watch src files changes
  -m, --minify           compress output. default: false
  -o, --output <file>    specify output filename
  -f, --format <format>  type of output (esm, amd, cjs, iife, umd, system), default: esm
  -h, --help             output usage information
  --prepare              auto configure package.json exports for building
  --external <mod>       specify an external dependency, separate by comma
  --no-external          do not bundle external dependencies
  --target <target>      js features target: swc target es versions. default: es2015
  --runtime <runtime>    build runtime (nodejs, browser). default: browser
  --env <env>            inlined process env variables, separate by comma. default: NODE_ENV
  --cwd <cwd>            specify current working directory
  --sourcemap            enable sourcemap generation, default: false
  --dts                  determine if need to generate types, default: false
`;
function help() {
    logger.log(helpMessage);
}
async function lintPackage(cwd) {
    // Not package.json detected, skip package linting
    if (!await hasPackageJson(cwd)) {
        return;
    }
    const { publint } = await import('publint');
    const { formatMessage } = await import('publint/utils');
    const { messages } = await publint({
        pkgDir: cwd,
        level: 'error'
    });
    const pkg = await getPackageMeta(cwd);
    for (const message of messages){
        console.log(formatMessage(message, pkg));
    }
}
function parseCliArgs(argv) {
    let args;
    args = arg__default.default({
        '--cwd': String,
        '--dts': Boolean,
        '--output': String,
        '--format': String,
        '--watch': Boolean,
        '--minify': Boolean,
        '--help': Boolean,
        '--version': Boolean,
        '--runtime': String,
        '--target': String,
        '--sourcemap': Boolean,
        '--env': String,
        '--external': String,
        '--no-external': Boolean,
        '--prepare': Boolean,
        '-h': '--help',
        '-v': '--version',
        '-w': '--watch',
        '-o': '--output',
        '-f': '--format',
        '-m': '--minify'
    }, {
        permissive: true,
        argv
    });
    const source = args._[0];
    const parsedArgs = {
        source,
        format: args['--format'],
        file: args['--output'],
        watch: args['--watch'],
        minify: args['--minify'],
        sourcemap: !!args['--sourcemap'],
        cwd: args['--cwd'],
        dts: args['--dts'],
        help: args['--help'],
        version: args['--version'],
        runtime: args['--runtime'],
        target: args['--target'],
        external: !!args['--no-external'] ? null : args['--external'],
        env: args['--env'],
        prepare: !!args['--prepare']
    };
    return parsedArgs;
}
async function run(args) {
    var _args_external;
    const { source, format, watch, minify, sourcemap, target, runtime, dts, env } = args;
    const cwd = args.cwd || process.cwd();
    const file = args.file ? path__default.default.resolve(cwd, args.file) : undefined;
    const bundleConfig = {
        dts,
        file,
        format,
        cwd,
        target,
        runtime,
        external: args.external === null ? null : ((_args_external = args.external) == null ? void 0 : _args_external.split(',')) || [],
        watch: !!watch,
        minify: !!minify,
        sourcemap: sourcemap === false ? false : true,
        env: (env == null ? void 0 : env.split(',')) || []
    };
    if (args.version) {
        return logger.log(version);
    }
    if (args.help) {
        return help();
    }
    if (args.prepare) {
        return await prepare(cwd);
    }
    const entry = source ? path__default.default.resolve(cwd, source) : '';
    try {
        await bunchee.bundle(entry, bundleConfig);
    } catch (err) {
        if (err.name === 'NOT_EXISTED') {
            help();
            return exit(err);
        }
        throw err;
    }
    // watching mode
    if (watch) {
        logger.log(`Watching assets in ${cwd}...`);
        return;
    }
    // build mode
    logger.log();
    paint('✓', 'green', `bunchee ${version} build completed`);
    await lintPackage(cwd);
}
async function main() {
    let params, error;
    try {
        params = parseCliArgs(process.argv.slice(2));
    } catch (err) {
        error = err;
    }
    if (error || !params) {
        if (!error) help();
        return exit(error);
    }
    await run(params);
}
main().catch(exit);
