Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs/promises');
var path = require('path');
var rollup = require('rollup');
var module$1 = require('module');
var require$$0 = require('tty');
var pluginWasm = require('@rollup/plugin-wasm');
var rollupPluginSwc3 = require('rollup-plugin-swc3');
var commonjs = require('@rollup/plugin-commonjs');
var json = require('@rollup/plugin-json');
var pluginNodeResolve = require('@rollup/plugin-node-resolve');
var replace = require('@rollup/plugin-replace');
var esmShim = require('@rollup/plugin-esm-shim');
var preserveDirectives = require('rollup-preserve-directives');
var pluginutils = require('@rollup/pluginutils');
var prettyBytes = require('pretty-bytes');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var fs__default = /*#__PURE__*/_interopDefault(fs);
var path__default = /*#__PURE__*/_interopDefault(path);
var require$$0__default = /*#__PURE__*/_interopDefault(require$$0);
var commonjs__default = /*#__PURE__*/_interopDefault(commonjs);
var json__default = /*#__PURE__*/_interopDefault(json);
var replace__default = /*#__PURE__*/_interopDefault(replace);
var esmShim__default = /*#__PURE__*/_interopDefault(esmShim);
var preserveDirectives__default = /*#__PURE__*/_interopDefault(preserveDirectives);
var prettyBytes__default = /*#__PURE__*/_interopDefault(prettyBytes);

const availableExtensions = [
    'js',
    'cjs',
    'mjs',
    'jsx',
    'ts',
    'tsx',
    'cts',
    'mts'
];
const nodeResolveExtensions = [
    '.mjs',
    '.cjs',
    '.js',
    '.json',
    '.node',
    '.jsx'
];
const availableExportConventions = [
    'react-server',
    'react-native',
    'edge-light'
];
const availableESExtensionsRegex = /\.(m|c)?[jt]sx?$/;
const SRC = 'src';
const dtsExtensionsMap = {
    js: 'd.ts',
    cjs: 'd.cts',
    mjs: 'd.mts'
};
const disabledWarnings = new Set([
    'MIXED_EXPORTS',
    'PREFER_NAMED_EXPORTS',
    'UNRESOLVED_IMPORT',
    'THIS_IS_UNDEFINED',
    'INVALID_ANNOTATION',
    'UNUSED_EXTERNAL_IMPORT'
]);

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

var picocolors = {exports: {}};

let tty = require$$0__default.default;
let isColorSupported = !("NO_COLOR" in process.env || process.argv.includes("--no-color")) && ("FORCE_COLOR" in process.env || process.argv.includes("--color") || process.platform === "win32" || tty.isatty(1) && process.env.TERM !== "dumb" || "CI" in process.env);
let formatter = (open, close, replace = open)=>(input)=>{
        let string = "" + input;
        let index = string.indexOf(close, open.length);
        return ~index ? open + replaceClose(string, close, replace, index) + close : open + string + close;
    };
let replaceClose = (string, close, replace, index)=>{
    let start = string.substring(0, index) + replace;
    let end = string.substring(index + close.length);
    let nextIndex = end.indexOf(close);
    return ~nextIndex ? start + replaceClose(end, close, replace, nextIndex) : start + end;
};
let createColors = (enabled = isColorSupported)=>({
        isColorSupported: enabled,
        reset: enabled ? (s)=>`\x1b[0m${s}\x1b[0m` : String,
        bold: enabled ? formatter("\x1b[1m", "\x1b[22m", "\x1b[22m\x1b[1m") : String,
        dim: enabled ? formatter("\x1b[2m", "\x1b[22m", "\x1b[22m\x1b[2m") : String,
        italic: enabled ? formatter("\x1b[3m", "\x1b[23m") : String,
        underline: enabled ? formatter("\x1b[4m", "\x1b[24m") : String,
        inverse: enabled ? formatter("\x1b[7m", "\x1b[27m") : String,
        hidden: enabled ? formatter("\x1b[8m", "\x1b[28m") : String,
        strikethrough: enabled ? formatter("\x1b[9m", "\x1b[29m") : String,
        black: enabled ? formatter("\x1b[30m", "\x1b[39m") : String,
        red: enabled ? formatter("\x1b[31m", "\x1b[39m") : String,
        green: enabled ? formatter("\x1b[32m", "\x1b[39m") : String,
        yellow: enabled ? formatter("\x1b[33m", "\x1b[39m") : String,
        blue: enabled ? formatter("\x1b[34m", "\x1b[39m") : String,
        magenta: enabled ? formatter("\x1b[35m", "\x1b[39m") : String,
        cyan: enabled ? formatter("\x1b[36m", "\x1b[39m") : String,
        white: enabled ? formatter("\x1b[37m", "\x1b[39m") : String,
        gray: enabled ? formatter("\x1b[90m", "\x1b[39m") : String,
        bgBlack: enabled ? formatter("\x1b[40m", "\x1b[49m") : String,
        bgRed: enabled ? formatter("\x1b[41m", "\x1b[49m") : String,
        bgGreen: enabled ? formatter("\x1b[42m", "\x1b[49m") : String,
        bgYellow: enabled ? formatter("\x1b[43m", "\x1b[49m") : String,
        bgBlue: enabled ? formatter("\x1b[44m", "\x1b[49m") : String,
        bgMagenta: enabled ? formatter("\x1b[45m", "\x1b[49m") : String,
        bgCyan: enabled ? formatter("\x1b[46m", "\x1b[49m") : String,
        bgWhite: enabled ? formatter("\x1b[47m", "\x1b[49m") : String
    });
picocolors.exports = createColors();
picocolors.exports.createColors = createColors;
var picocolorsExports = picocolors.exports;
var pc = /*@__PURE__*/ getDefaultExportFromCjs(picocolorsExports);

const defaultColorFn = (text)=>text;
function color(prefixColor) {
    return pc.isColorSupported ? pc[prefixColor] : defaultColorFn;
}
const logger = {
    log (...arg) {
        console.log(...arg);
    },
    warn (...arg) {
        console.warn(color('yellow')('⚠️'), ...arg);
    },
    error (...arg) {
        console.error(color('red')('⨯'), ...arg);
    },
    info (...arg) {
        console.log(color('green')('✓'), ...arg);
    }
};

function exit(err) {
    logger.error(err);
    process.exit(1);
}
async function getPackageMeta(cwd) {
    const pkgFilePath = path__default.default.resolve(cwd, 'package.json');
    let targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(await fs__default.default.readFile(pkgFilePath, {
            encoding: 'utf-8'
        }));
    } catch (_) {}
    return targetPackageJson;
}
async function fileExists(filePath) {
    try {
        await fs__default.default.access(filePath);
        return true;
    } catch (err) {
        if (err.code === 'ENOENT') {
            return false;
        }
        throw err;
    }
}
// . -> pkg name
// ./lite -> <pkg name>/lite
function getExportPath(pkg, cwd, exportName) {
    const name = pkg.name || path__default.default.basename(cwd);
    if (exportName === '.' || !exportName) return name;
    return path__default.default.join(name, exportName);
}
const isNotNull = (n)=>Boolean(n);
function resolveSourceFile(cwd, filename) {
    return path__default.default.resolve(cwd, SRC, filename);
}
async function findSourceEntryFile(cwd, exportPath, exportTypeSuffix, ext) {
    const filename = resolveSourceFile(cwd, `${exportPath}${exportTypeSuffix ? `.${exportTypeSuffix}` : ''}.${ext}`);
    if (await fileExists(filename)) {
        return filename;
    }
    const subFolderIndexFilename = resolveSourceFile(cwd, `${exportPath}/index${exportTypeSuffix ? `.${exportTypeSuffix}` : ''}.${ext}`);
    if (await fileExists(subFolderIndexFilename)) {
        return subFolderIndexFilename;
    }
    return undefined;
}
// Map '.' -> './index.[ext]'
// Map './lite' -> './lite.[ext]'
// Return undefined if no match or if it's package.json exports
async function getSourcePathFromExportPath(cwd, exportPath, exportType) {
    for (const ext of availableExtensions){
        // ignore package.json
        if (exportPath.endsWith('package.json')) return;
        if (exportPath === '.') exportPath = './index';
        // Find convention-based source file for specific export types
        // $binary represents `pkg.bin`
        if (availableExportConventions.includes(exportType) && exportType !== '$binary') {
            const filename = await findSourceEntryFile(cwd, exportPath, exportType, ext);
            if (filename) return filename;
        }
        const filename = await findSourceEntryFile(cwd, exportPath, null, ext);
        if (filename) return filename;
    }
    return;
}
// Unlike path.basename, forcedly removing extension
function filePathWithoutExtension(file) {
    return file ? file.replace(new RegExp(`${path__default.default.extname(file)}$`), '') : undefined;
}
const nonNullable = (n)=>Boolean(n);
const hasAvailableExtension = (filename)=>availableExtensions.includes(path__default.default.extname(filename).slice(1));
const hasCjsExtension = (filename)=>path__default.default.extname(filename) === '.cjs';

let hasLoggedTsWarning = false;
function resolveTypescript(cwd) {
    let ts;
    const m = new module$1.Module('', undefined);
    m.paths = module$1.Module._nodeModulePaths(cwd);
    try {
        ts = m.require('typescript');
    } catch (e) {
        console.error(e);
        if (!hasLoggedTsWarning) {
            hasLoggedTsWarning = true;
            exit('Could not load TypeScript compiler. Try to install `typescript` as dev dependency');
        }
    }
    return ts;
}
async function resolveTsConfig(cwd) {
    let tsCompilerOptions = {};
    let tsConfigPath;
    tsConfigPath = path.resolve(cwd, 'tsconfig.json');
    if (await fileExists(tsConfigPath)) {
        const ts = resolveTypescript(cwd);
        const basePath = tsConfigPath ? path.dirname(tsConfigPath) : cwd;
        const tsconfigJSON = ts.readConfigFile(tsConfigPath, ts.sys.readFile).config;
        tsCompilerOptions = ts.parseJsonConfigFileContent(tsconfigJSON, ts.sys, basePath).options;
    } else {
        return null;
    }
    return {
        tsCompilerOptions,
        tsConfigPath
    };
}
async function convertCompilerOptions(cwd, json) {
    const ts = resolveTypescript(cwd);
    return ts.convertCompilerOptionsFromJson(json, './');
}

function minifyCSS(content) {
    return content.replace(/\/\*[\s\S]*?\*\/|([^:]|^)\/\/.*$|(?:^|\s)(\s+)|\s*([\{\};,:])\s*|\s+(!)\s+/g, (match, p1, p2, p3, p4)=>{
        if (p1) return p1 === ' ' ? '' : p1;
        if (p2) return ' ';
        if (p3) return p3;
        if (p4) return '!';
    });
}
const helpers = {
    cssImport: {
        // have to assign r.type = 'text/css' to make it work in Safari
        global: `\
function __insertCSS(code) {
  if (!code || typeof document == 'undefined') return
  let head = document.head || document.getElementsByTagName('head')[0]
  let style = document.createElement('style')
  style.type = 'text/css'
  head.appendChild(style)
  ;style.styleSheet ? (style.styleSheet.cssText = code) : style.appendChild(document.createTextNode(code))
}
`,
        create (code) {
            return `__insertCSS(${JSON.stringify(code)});`;
        }
    },
    cssAssertionImport: {
        global: '',
        create (code) {
            return `\
const sheet = new CSSStyleSheet()
sheet.replaceSync(${JSON.stringify(code)})
export default sheet`;
        }
    }
};
function inlineCss(options) {
    const cssIds = new Set();
    var _options_exclude;
    const filter = pluginutils.createFilter([
        '**/*.css'
    ], (_options_exclude = options.exclude) != null ? _options_exclude : []);
    // Follow up for rollup 4 for better support of assertion support https://github.com/rollup/rollup/issues/4818
    return {
        name: 'inline-css',
        transform (code, id) {
            if (!filter(id)) return;
            if (options.skip) return '';
            const cssCode = minifyCSS(code);
            cssIds.add(id);
            return {
                code: helpers.cssImport.create(cssCode),
                map: {
                    mappings: ''
                }
            };
        },
        renderChunk (code, options) {
            const dependenciesIds = this.getModuleIds();
            let foundCss = false;
            for (const depId of dependenciesIds){
                if (depId && cssIds.has(depId)) {
                    foundCss = true;
                    break;
                }
            }
            if (!foundCss) return;
            return {
                code: `${helpers.cssImport.global}\n${code}`,
                map: {
                    mappings: ''
                }
            };
        }
    };
}

function rawContent({ exclude }) {
    const filter = pluginutils.createFilter([
        '**/*.data',
        '**/*.txt'
    ], exclude);
    return {
        name: 'string',
        transform (code, id) {
            if (filter(id)) {
                return {
                    code: `const data = ${JSON.stringify(code)};\nexport default data;`,
                    map: null
                };
            }
            return null;
        }
    };
}

// Alias entries to import path
// e.g.
// For a resolved file, if it's one of the entries,
// aliases it as export path, such as <absolute file> -> <pkg>/<export path>
function aliasEntries({ entries }) {
    return {
        name: 'alias',
        resolveId: {
            async handler (source, importer, options) {
                const resolvedId = await this.resolve(source, importer, options);
                if (resolvedId != null) {
                    const aliasedId = entries[resolvedId.id];
                    if (aliasedId != null) {
                        return {
                            id: aliasedId,
                            external: true
                        };
                    }
                }
                return null;
            }
        }
    };
}

function prependDirectives() {
    return {
        name: 'prependDirective',
        transform: {
            order: 'post',
            handler (code, id) {
                var _moduleInfo_meta;
                const moduleInfo = this.getModuleInfo(id);
                if (moduleInfo == null ? void 0 : (_moduleInfo_meta = moduleInfo.meta) == null ? void 0 : _moduleInfo_meta.preserveDirectives) {
                    const firstDirective = moduleInfo.meta.preserveDirectives.directives[0];
                    if (firstDirective) {
                        const directive = firstDirective.value;
                        const directiveCode = `'${directive}';`;
                        return directiveCode + '\n' + code;
                    }
                }
                return null;
            }
        }
    };
}

function getTypings(pkg) {
    return pkg.types || pkg.typings;
}
function getDistPath(distPath, cwd) {
    return path.resolve(cwd, distPath);
}
// Reached the end of the export path
function isExportLike(field) {
    if (typeof field === 'string') return true;
    return Object.entries(field).every(// Every value is string and key is not start with '.'
    ([key, value])=>typeof value === 'string' && !key.startsWith('.'));
}
function constructFullExportCondition(exportCondition, packageType) {
    let fullExportCond;
    if (typeof exportCondition === 'string') {
        const exportType = getExportTypeFromFile(exportCondition, packageType);
        fullExportCond = {
            [exportType]: exportCondition
        };
    } else {
        const exportTypes = Object.keys(exportCondition);
        fullExportCond = {};
        exportTypes.forEach((exportType)=>{
            const condition = exportCondition[exportType];
            // Filter out nullable value
            if (condition) {
                fullExportCond[exportType] = condition;
            }
        });
    }
    return fullExportCond;
}
function joinRelativePath(...segments) {
    let result = path.join(...segments);
    // If the first segment starts with '.', ensure the result does too.
    if (segments[0] === '.' && !result.startsWith('.')) {
        result = './' + result;
    }
    return result;
}
function findExport(exportPath, exportCondition, paths, packageType, currentPath) {
    // Skip `types` field, it cannot be the entry point
    if (exportPath === 'types') return;
    if (isExportLike(exportCondition)) {
        const fullExportCondition = constructFullExportCondition(exportCondition, packageType);
        if (exportPath.startsWith('.')) {
            paths[exportPath] = {
                ...paths[exportPath],
                ...fullExportCondition
            };
        } else {
            // exportPath is exportType, import, require, ...
            // merge to currentPath
            paths[currentPath] = {
                ...paths[currentPath],
                [exportPath]: fullExportCondition.default
            };
        }
        return;
    }
    Object.keys(exportCondition).forEach((subpath)=>{
        if (subpath.startsWith('.')) {
            // subpath is actual export path, ./a, ./b, ...
            const nestedExportPath = joinRelativePath(currentPath, subpath);
            const nestedExportCondition = exportCondition[subpath];
            findExport(nestedExportPath, nestedExportCondition, paths, packageType, nestedExportPath);
        } else {
            // subpath is exportType, import, require, ...
            const exportType = subpath;
            const defaultPath = typeof exportCondition[subpath] === 'object' ? exportCondition[subpath].default : exportCondition[subpath];
            const nestedExportCondition = {
                [exportType]: defaultPath
            };
            findExport(exportPath, nestedExportCondition, paths, packageType, currentPath);
        }
    });
}
/**
 *
 * Convert package.exports field to paths mapping
 * Example
 *
 * Input:
 * {
 *   "./sub": {
 *     "import": {
 *       "types": "./sub.js",
 *       "default": "./sub.cjs",
 *     }
 *   }
 * }
 *
 * Output:
 * {
 *   "./sub": {
 *     "import": "./sub.js",
 *     "require": "./sub.cjs",
 *     "types": "./sub.d.ts",
 *   }
 * }
 *
 */ function parseExport(exportsCondition, packageType) {
    const paths = {};
    const initialPath = '.';
    if (typeof exportsCondition === 'string') {
        paths[initialPath] = constructFullExportCondition(exportsCondition, packageType);
    } else if (typeof exportsCondition === 'object') {
        if (isExportLike(exportsCondition)) {
            paths[initialPath] = constructFullExportCondition(exportsCondition, packageType);
        } else {
            Object.keys(exportsCondition).forEach((key)=>{
                const exportCondition = exportsCondition[key];
                findExport(key, exportCondition, paths, packageType, initialPath);
            });
        }
    }
    return paths;
}
/**
 * Get package exports paths
 *
 * Example:
 *
 * ```json
 * {
 *  "exports": {
 *    ".": {
 *      "require": "./dist/index.cjs",
 *      "module": "./dist/index.esm.js",
 *      "default": "./dist/index.esm.js"
 *    },
 *    "./foo": {
 *      "require": "./dist/foo.cjs",
 *      "module": "./dist/foo.esm.js",
 *      "default": "./dist/foo.esm.js"
 *   }
 * }
 *
 * ```
 *
 * will be parsed to:
 *
 * ```js
 * {
 *   '.': {
 *     main: './dist/index.cjs',
 *     module: './dist/index.esm.js',
 *     export: './dist/index.esm.js'
 *   },
 *   './foo': {
 *     main: './dist/foo.cjs',
 *     module: './dist/foo.esm.js',
 *     export: './dist/foo.esm.js'
 *   }
 *
 *
 * pkg.main and pkg.module will be added to ['.'] if exists
 */ function getExportPaths(pkg, pkgType, resolvedWildcardExports) {
    var _pathsMap_;
    let pathsMap = {};
    const packageType = pkgType != null ? pkgType : getPackageType(pkg);
    const isEsmPackage = isESModulePackage(packageType);
    const exportsConditions = resolvedWildcardExports != null ? resolvedWildcardExports : pkg.exports;
    if (exportsConditions) {
        const paths = parseExport(exportsConditions, packageType);
        pathsMap = {
            ...pathsMap,
            ...paths
        };
    }
    if (isEsmPackage && pkg.main && hasCjsExtension(pkg.main)) {
        exit('Cannot export main field with .cjs extension in ESM package, only .mjs and .js extensions are allowed');
    }
    // main export '.' from main/module/typings
    const defaultMainExport = constructFullExportCondition({
        [isEsmPackage ? 'import' : 'require']: pkg.main,
        module: pkg.module,
        types: getTypings(pkg)
    }, packageType);
    if (!isEsmPackage && ((_pathsMap_ = pathsMap['.']) == null ? void 0 : _pathsMap_['require'])) {
        // pathsMap's exports.require are prioritized.
        defaultMainExport['require'] = pathsMap['.']['require'];
    }
    // Merge the main export into '.' paths
    const mainExport = {
        ...pathsMap['.'],
        ...defaultMainExport
    };
    // main export is not empty
    if (Object.keys(mainExport).length > 0) {
        pathsMap['.'] = {
            ...pathsMap['.'],
            ...mainExport
        };
    }
    return pathsMap;
}
const getExportTypeDist = (parsedExportCondition, cwd)=>{
    const existed = new Set();
    const exportTypes = Object.keys(parsedExportCondition.export);
    for (const key of exportTypes){
        if (key === 'module') {
            continue;
        }
        const filePath = parsedExportCondition.export[key];
        if (key === 'types') {
            const typeFile = getDistPath(filePath, cwd);
            if (existed.has(typeFile)) {
                continue;
            }
            existed.add(typeFile);
            continue;
        }
        const ext = path.extname(filePath).slice(1);
        const typeFile = getDistPath(`${filePathWithoutExtension(filePath) || ''}.${dtsExtensionsMap[ext]}`, cwd);
        if (existed.has(typeFile)) {
            continue;
        }
        existed.add(typeFile);
    }
    return Array.from(existed);
};
function getPackageType(pkg) {
    return pkg.type || 'commonjs';
}
function isESModulePackage(packageType) {
    return packageType === 'module';
}
function constructDefaultExportCondition(value, packageType) {
    const isEsmPackage = isESModulePackage(packageType);
    let exportCondition;
    if (typeof value === 'string') {
        const types = getTypings(value);
        exportCondition = {
            [isEsmPackage ? 'import' : 'require']: value,
            ...types && {
                types
            }
        };
    } else {
        exportCondition = value;
    }
    return constructFullExportCondition(exportCondition, packageType);
}
function isEsmExportName(name, ext) {
    return [
        'import',
        'module'
    ].includes(name) || ext === 'mjs';
}
function isCjsExportName(pkg, name, ext) {
    const isESModule = isESModulePackage(pkg.type);
    return !isESModule && [
        'require',
        'main',
        'node'
    ].includes(name) && ext !== 'mjs' || ext === 'cjs';
}
function getExportConditionDist(pkg, parsedExportCondition, cwd) {
    const dist = [];
    const existed = new Set();
    const exportTypes = Object.keys(parsedExportCondition.export);
    for (const exportType of exportTypes){
        if (exportType === 'types') {
            continue;
        }
        const filePath = parsedExportCondition.export[exportType];
        const ext = path.extname(filePath).slice(1);
        const relativePath = parsedExportCondition.export[exportType];
        const distFile = getDistPath(relativePath, cwd);
        let format = 'esm';
        if (isCjsExportName(pkg, exportType, ext)) {
            format = 'cjs';
        }
        // Deduplicate the same path jobs
        // TODO: detect conflicts paths but with different format
        if (existed.has(distFile)) {
            continue;
        }
        existed.add(distFile);
        dist.push({
            format,
            file: distFile
        });
    }
    if (dist.length === 0 && !pkg.bin) {
        const defaultFormat = isESModulePackage(pkg.type) ? 'esm' : 'cjs';
        dist.push({
            format: defaultFormat,
            file: getDistPath('dist/index.js', cwd)
        });
    }
    return dist;
}
function getTypeFilePath(entryFilePath, exportCondition, cwd) {
    const name = filePathWithoutExtension(entryFilePath);
    const firstDistPath = exportCondition ? Object.values(exportCondition.export)[0] : undefined;
    const exportName = (exportCondition == null ? void 0 : exportCondition.name) || 'index';
    return entryFilePath ? name + '.d.ts' : path.resolve(firstDistPath ? path.dirname(firstDistPath) : path.join(cwd, 'dist'), (exportName === '.' ? 'index' : exportName) + '.d.ts');
}
function getExportTypeFromFile(filename, pkgType) {
    const isESModule = isESModulePackage(pkgType);
    const isCjsExt = filename.endsWith('.cjs');
    const isEsmExt = filename.endsWith('.mjs');
    const exportType = isEsmExt ? 'import' : isCjsExt ? 'require' : isESModule ? 'import' : 'require';
    return exportType;
}

const swcMinifyOptions = {
    compress: true,
    format: {
        comments: 'some'
    },
    mangle: {
        toplevel: true
    }
};
function getBuildEnv(envs) {
    if (!envs.includes('NODE_ENV')) {
        envs.push('NODE_ENV');
    }
    const envVars = envs.reduce((acc, key)=>{
        const value = process.env[key];
        if (typeof value !== 'undefined') {
            acc['process.env.' + key] = JSON.stringify(value);
        }
        return acc;
    }, {});
    return envVars;
}
/**
 * return { '<absolute source path>': '<pkg>/<export>' }
 */ function getReversedAlias(entries) {
    const alias = {};
    for (const [entryImportPath, exportCondition] of Object.entries(entries)){
        const exportType = entryImportPath.split('.')[1] // e.g. index.react-server, pick react-server
        ;
        if (!exportType) {
            alias[exportCondition.source] = entryImportPath;
        }
    }
    return alias;
}
async function buildInputConfig(entry, options, buildContext, dts) {
    const { entries, pkg, cwd, tsOptions: { tsConfigPath, tsCompilerOptions }, pluginContext } = buildContext;
    const hasNoExternal = options.external === null;
    var _options_external;
    const externals = hasNoExternal ? [] : [
        pkg.peerDependencies,
        pkg.dependencies,
        pkg.peerDependenciesMeta
    ].filter((n)=>Boolean(n)).map((o)=>Object.keys(o)).reduce((a, b)=>a.concat(b), []).concat((_options_external = options.external) != null ? _options_external : []);
    for (const [exportImportPath, exportCondition] of Object.entries(entries)){
        const entryFilePath = exportCondition.source;
        if (entryFilePath !== entry) {
            externals.push(exportImportPath);
            externals.push(entryFilePath);
        }
    }
    const { useTypescript, runtime, target: jscTarget, minify: shouldMinify } = options;
    const hasSpecifiedTsTarget = Boolean(tsCompilerOptions.target && tsConfigPath);
    const swcParserConfig = {
        syntax: useTypescript ? 'typescript' : 'ecmascript',
        [useTypescript ? 'tsx' : 'jsx']: true,
        exportDefaultFrom: true
    };
    const swcOptions = {
        jsc: {
            ...!hasSpecifiedTsTarget && {
                target: jscTarget
            },
            loose: true,
            externalHelpers: false,
            parser: swcParserConfig,
            ...shouldMinify && {
                minify: {
                    ...swcMinifyOptions,
                    sourceMap: options.sourcemap
                }
            }
        },
        sourceMaps: options.sourcemap,
        inlineSourcesContent: false,
        isModule: true
    };
    const sizePlugin = pluginContext.outputState.plugin(cwd);
    // common plugins for both dts and ts assets that need to be processed
    const commonPlugins = [
        sizePlugin,
        aliasEntries({
            entries: {
                ...pluginContext.entriesAlias,
                // Do not alias current alias of package
                [entry]: null
            }
        })
    ];
    const typesPlugins = [
        ...commonPlugins,
        inlineCss({
            skip: true
        })
    ];
    if (useTypescript) {
        const { options: overrideResolvedTsOptions } = await convertCompilerOptions(cwd, {
            declaration: true,
            noEmit: false,
            noEmitOnError: true,
            emitDeclarationOnly: true,
            checkJs: false,
            declarationMap: false,
            skipLibCheck: true,
            target: 'ESNext',
            // Some react types required this to be false by default.
            // Some type package like express might need this as it has other dependencies.
            // Let users able to toggle this in tsconfig.
            preserveSymlinks: 'preserveSymlinks' in tsCompilerOptions ? tsCompilerOptions.preserveSymlinks : false,
            ...!tsCompilerOptions.jsx ? {
                jsx: 'react-jsx'
            } : undefined,
            // error TS5074: Option '--incremental' can only be specified using tsconfig, emitting to single
            // file or when option '--tsBuildInfoFile' is specified.
            incremental: false
        });
        const dtsPlugin = require('rollup-plugin-dts').default({
            tsconfig: tsConfigPath,
            compilerOptions: overrideResolvedTsOptions
        });
        typesPlugins.push(dtsPlugin);
    }
    const plugins = (dts ? typesPlugins : [
        ...commonPlugins,
        inlineCss({
            exclude: /node_modules/
        }),
        rawContent({
            exclude: /node_modules/
        }),
        esmShim__default.default(),
        preserveDirectives__default.default(),
        prependDirectives(),
        replace__default.default({
            values: getBuildEnv(options.env || []),
            preventAssignment: true
        }),
        pluginNodeResolve.nodeResolve({
            preferBuiltins: runtime === 'node',
            extensions: nodeResolveExtensions
        }),
        commonjs__default.default({
            exclude: options.external || null
        }),
        json__default.default(),
        pluginWasm.wasm(),
        rollupPluginSwc3.swc({
            include: availableESExtensionsRegex,
            exclude: 'node_modules',
            tsconfig: tsConfigPath,
            ...swcOptions
        })
    ]).filter(isNotNull);
    return {
        input: entry,
        external (id) {
            return externals.some((name)=>id === name || id.startsWith(name + '/'));
        },
        plugins,
        treeshake: {
            propertyReadSideEffects: false
        },
        onwarn (warning, warn) {
            const code = warning.code || '';
            // Some may not have types, like CLI binary
            if (dts && code === 'EMPTY_BUNDLE') return;
            if (disabledWarnings.has(code)) return;
            // If the circular dependency warning is from node_modules, ignore it
            if (code === 'CIRCULAR_DEPENDENCY' && /Circular dependency: node_modules/.test(warning.message)) {
                return;
            }
            warn(warning);
        }
    };
}
function hasEsmExport(exportPaths, tsCompilerOptions) {
    let hasEsm = false;
    for(const key in exportPaths){
        const exportInfo = exportPaths[key];
        const exportInfoEntries = Object.entries(exportInfo);
        if (exportInfoEntries.some(([exportType, file])=>isEsmExportName(exportType, file))) {
            hasEsm = true;
            break;
        }
    }
    return Boolean(hasEsm || (tsCompilerOptions == null ? void 0 : tsCompilerOptions.esModuleInterop));
}
function getModuleLater(moduleMeta) {
    const directives = (moduleMeta.preserveDirectives || {
        directives: []
    }).directives.map((d)=>d.replace(/^use /, '')).filter((d)=>d !== 'strict');
    const moduleLayer = directives[0];
    return moduleLayer;
}
// dependencyGraphMap: Map<subModuleId, Set<entryParentId>>
function createSplitChunks(dependencyGraphMap, entryFiles) {
    // If there's existing chunk being splitted, and contains a layer { <id>: <chunkGroup> }
    const splitChunksGroupMap = new Map();
    return function splitChunks(id, ctx) {
        const moduleInfo = ctx.getModuleInfo(id);
        if (!moduleInfo) {
            return;
        }
        const { isEntry } = moduleInfo;
        const moduleMeta = moduleInfo.meta;
        const moduleLayer = getModuleLater(moduleMeta);
        // Collect the sub modules of the entry, if they're having layer, and the same layer with the entry, push them to the dependencyGraphMap.
        if (isEntry) {
            const subModuleIds = ctx.getModuleIds();
            for (const subId of subModuleIds){
                const subModuleInfo = ctx.getModuleInfo(subId);
                if (!subModuleInfo) {
                    continue;
                }
                const subModuleLayer = getModuleLater(moduleMeta);
                if (subModuleLayer === moduleLayer) {
                    if (!dependencyGraphMap.has(subId)) {
                        dependencyGraphMap.set(subId, new Set());
                    }
                    dependencyGraphMap.get(subId).add([
                        id,
                        moduleLayer
                    ]);
                }
            }
        }
        // If current module has a layer, and it's not an entry
        if (moduleLayer && !isEntry) {
            // If the module is imported by the entry:
            // when the module layer is same as entry layer, keep it as part of entry and don't split it;
            // when the module layer is different from entry layer, split the module into a separate chunk as a separate boundary.
            if (dependencyGraphMap.has(id)) {
                const parentModuleIds = Array.from(dependencyGraphMap.get(id));
                const isImportFromOtherEntry = parentModuleIds.some(([id])=>{
                    // If other entry is dependency of this entry
                    if (entryFiles.has(id)) {
                        const entryModuleInfo = ctx.getModuleInfo(id);
                        const entryModuleLayer = getModuleLater(entryModuleInfo ? entryModuleInfo.meta : {});
                        return entryModuleLayer === moduleLayer;
                    }
                    return false;
                });
                if (isImportFromOtherEntry) return;
                const isPartOfCurrentEntry = parentModuleIds.every(([, layer])=>layer === moduleLayer);
                if (isPartOfCurrentEntry) {
                    if (splitChunksGroupMap.has(id)) {
                        return splitChunksGroupMap.get(id);
                    }
                    return;
                }
                const chunkName = path__default.default.basename(id, path__default.default.extname(id));
                const chunkGroup = `${chunkName}-${moduleLayer}`;
                splitChunksGroupMap.set(id, chunkGroup);
                return chunkGroup;
            }
        }
        return;
    };
}
function buildOutputConfigs(options, exportCondition, buildContext, dts) {
    const { format } = options;
    const { entries, pkg, exportPaths, cwd, tsOptions: { tsCompilerOptions }, pluginContext } = buildContext;
    // Add esm mark and interop helper if esm export is detected
    const useEsModuleMark = hasEsmExport(exportPaths, tsCompilerOptions);
    const typings = getTypings(pkg);
    const file = options.file && path.resolve(cwd, options.file);
    const dtsDir = typings ? path.dirname(path.resolve(cwd, typings)) : path.resolve(cwd, 'dist');
    const name = filePathWithoutExtension(file);
    // TODO: simplify dts file name detection
    const dtsFile = file ? file : exportCondition.export['types'] ? path.resolve(cwd, exportCondition.export['types']) : path.resolve(dtsDir, (exportCondition.name === '.' ? 'index' : exportCondition.name) + '.d.ts');
    const dtsPathConfig = {
        dir: dtsFile ? path.dirname(dtsFile) : dtsDir
    };
    const outputFile = dtsFile || file;
    const entryFiles = new Set(Object.values(entries).map((entry)=>entry.source));
    return {
        name: pkg.name || name,
        ...dts ? dtsPathConfig : {
            dir: path.dirname(outputFile)
        },
        format,
        exports: 'named',
        esModule: useEsModuleMark || 'if-default-prop',
        interop: 'auto',
        freeze: false,
        strict: false,
        sourcemap: options.sourcemap,
        manualChunks: createSplitChunks(pluginContext.moduleDirectiveLayerMap, entryFiles),
        chunkFileNames: '[name]-[hash].js',
        // By default in rollup, when creating multiple chunks, transitive imports of entry chunks
        // will be added as empty imports to the entry chunks. Disable to avoid imports hoist outside of boundaries
        hoistTransitiveImports: false,
        entryFileNames: path.basename(outputFile)
    };
}
async function buildEntryConfig(bundleConfig, pluginContext, dts) {
    const configs = [];
    const { entries } = pluginContext;
    for (const exportCondition of Object.values(entries)){
        const rollupConfig = buildConfig(bundleConfig, exportCondition, pluginContext, dts);
        configs.push(rollupConfig);
    }
    return await Promise.all(configs);
}
/*
 * build configs for each entry from package exports
 *
 * return { <pkg>/<export>: { input: InputOptions, output: OutputOptions[] }
 */ async function collectEntries(pkg, entryPath, exportPaths, cwd) {
    const entries = {};
    async function collectEntry(// export type, e.g. react-server, edge-light those special cases required suffix
    exportType, exportCondRef, // export name, e.g. ./<export-path> in exports field of package.json
    entryExport) {
        let exportCondForType = {
            ...exportCondRef
        };
        // Special cases of export type, only pass down the exportPaths for the type
        if (availableExportConventions.includes(exportType)) {
            exportCondForType = {
                [entryExport]: exportCondRef[exportType]
            };
        // Basic export type, pass down the exportPaths with erasing the special ones
        } else {
            for (const exportType of availableExportConventions){
                delete exportCondForType[exportType];
            }
        }
        let source = entryPath;
        if (source) {
            source = resolveSourceFile(cwd, source);
        } else {
            source = await getSourcePathFromExportPath(cwd, entryExport, exportType);
        }
        if (!source) {
            return undefined;
        }
        const exportCondition = {
            source,
            name: entryExport,
            export: exportCondForType
        };
        const nameWithExportPath = pkg.name ? path__default.default.join(pkg.name, exportCondition.name) : exportCondition.name;
        const needsDelimiter = !nameWithExportPath.endsWith('.') && exportType;
        const entryImportPath = nameWithExportPath + (needsDelimiter ? '.' : '') + exportType;
        entries[entryImportPath] = exportCondition;
    }
    const binaryExports = pkg.bin;
    if (binaryExports) {
        // binDistPaths: [ [ 'bin1', './dist/bin1.js'], [ 'bin2', './dist/bin2.js'] ]
        const binPairs = typeof binaryExports === 'string' ? [
            [
                'bin',
                binaryExports
            ]
        ] : Object.keys(binaryExports).map((key)=>[
                path.join('bin', key),
                binaryExports[key]
            ]);
        const binExportPaths = binPairs.reduce((acc, [binName, binDistPath])=>{
            const exportType = getExportTypeFromFile(binDistPath, pkg.type);
            acc[binName] = {
                [exportType]: binDistPath
            };
            return acc;
        }, {});
        for (const [binName] of binPairs){
            const source = await getSourcePathFromExportPath(cwd, binName, '$binary');
            if (!source) {
                logger.warn(`Cannot find source file for ${binName}`);
                continue;
            }
            const binEntryPath = await resolveSourceFile(cwd, source);
            entries[binName] = {
                source: binEntryPath,
                name: binName,
                export: binExportPaths[binName]
            };
        }
    }
    const collectEntriesPromises = Object.keys(exportPaths).map(async (entryExport)=>{
        const exportCond = exportPaths[entryExport];
        if (entryExport.startsWith('.')) {
            await collectEntry('', exportCond, entryExport);
            for (const exportType of availableExportConventions){
                if (exportCond[exportType]) {
                    await collectEntry(exportType, exportCond, entryExport);
                }
            }
        }
    });
    await Promise.all(collectEntriesPromises);
    return entries;
}
async function buildConfig(bundleConfig, exportCondition, pluginContext, dts) {
    const { file } = bundleConfig;
    const { pkg, cwd, tsOptions } = pluginContext;
    const useTypescript = Boolean(tsOptions.tsConfigPath);
    const options = {
        ...bundleConfig,
        useTypescript
    };
    const entry = exportCondition.source;
    const inputOptions = await buildInputConfig(entry, options, pluginContext, dts);
    const outputExports = getExportConditionDist(pkg, exportCondition, cwd);
    let outputConfigs = [];
    // Generate dts job - single config
    if (dts) {
        const typeOutputExports = getExportTypeDist(exportCondition, cwd);
        outputConfigs = typeOutputExports.map((typeFile)=>buildOutputConfigs({
                ...bundleConfig,
                format: 'es',
                useTypescript,
                file: typeFile
            }, exportCondition, pluginContext, dts));
    } else {
        // multi outputs with specified format
        outputConfigs = outputExports.map((exportDist)=>{
            return buildOutputConfigs({
                ...bundleConfig,
                file: exportDist.file,
                format: exportDist.format,
                useTypescript
            }, exportCondition, pluginContext, dts);
        });
        // CLI output option is always prioritized
        if (file) {
            var _outputExports_;
            const fallbackFormat = (_outputExports_ = outputExports[0]) == null ? void 0 : _outputExports_.format;
            outputConfigs = [
                buildOutputConfigs({
                    ...bundleConfig,
                    file,
                    format: bundleConfig.format || fallbackFormat,
                    useTypescript
                }, exportCondition, pluginContext, dts)
            ];
        }
    }
    return {
        input: inputOptions,
        output: outputConfigs,
        exportName: exportCondition.name || '.'
    };
}

function relativify(path) {
    return path.startsWith('.') ? path : `./${path}`;
}

function createOutputState({ entries }) {
    const sizeStats = new Map();
    function addSize({ fileName, size, sourceFileName, exportPath }) {
        if (!sizeStats.has(exportPath)) {
            sizeStats.set(exportPath, []);
        }
        const distFilesStats = sizeStats.get(exportPath);
        if (distFilesStats) {
            distFilesStats.push([
                fileName,
                sourceFileName,
                size
            ]);
        }
    }
    const reversedMapping = new Map();
    Object.entries(entries).forEach(([resolvedExportName, entry])=>{
        reversedMapping.set(entry.source, resolvedExportName);
    });
    return {
        plugin: (cwd)=>{
            return {
                name: 'collect-sizes',
                writeBundle (options, bundle) {
                    const dir = options.dir || path__default.default.dirname(options.file);
                    Object.entries(bundle).forEach(([fileName, chunk])=>{
                        const filePath = path__default.default.join(dir, fileName);
                        if (chunk.type !== 'chunk') {
                            return;
                        }
                        const size = chunk.code.length;
                        const sourceFileName = chunk.facadeModuleId || '';
                        const exportPath = reversedMapping.get(sourceFileName) || '.';
                        addSize({
                            fileName: path__default.default.isAbsolute(cwd) ? path__default.default.relative(cwd, filePath) : filePath,
                            size,
                            sourceFileName,
                            exportPath
                        });
                    });
                }
            };
        },
        getSizeStats () {
            return sizeStats;
        }
    };
}
function isBin(filename) {
    return filename === 'bin' || filename.startsWith('bin/');
}
function isTypeFile(filename) {
    return filename.endsWith('.d.ts') || filename.endsWith('.d.mts') || filename.endsWith('.d.cts');
}
function normalizeExportName(exportName) {
    const isBinary = isBin(exportName);
    let result = exportName;
    const isSubpathExport = exportName.includes('/');
    const isSpecialExport = exportName.includes('.');
    if (isBinary) {
        result = (exportName.replace(/bin(\/|$)/, '') || '.') + ' (bin)';
    } else if (isSubpathExport || isSpecialExport) {
        const subExportName = exportName.split('/')[1] || exportName;
        if (subExportName.includes('.') && subExportName !== '.') {
            const [originExportName, specialCondition] = subExportName.split('.');
            result = (isSubpathExport ? relativify(originExportName) : '.') + ' (' + specialCondition + ')';
        } else {
            result = isSubpathExport ? relativify(subExportName) : '.';
        }
    } else {
        result = '.';
    }
    return result;
}
function getExportNameWithoutExportCondition(exportName) {
    return exportName.includes('.') ? exportName.split('.')[0] : exportName;
}
function logOutputState(sizeCollector) {
    const stats = sizeCollector.getSizeStats();
    const allFileNameLengths = Array.from(stats.values()).flat(1).map(([filename])=>filename.length);
    const maxFilenameLength = Math.max(...allFileNameLengths);
    const statsArray = [
        ...stats.entries()
    ].sort(([a], [b])=>{
        const comp = getExportNameWithoutExportCondition(a).length - getExportNameWithoutExportCondition(b).length;
        return comp === 0 ? a.localeCompare(b) : comp;
    });
    const maxLengthOfExportName = Math.max(...statsArray.map(([exportName])=>normalizeExportName(exportName).length));
    console.log(pc.underline('Exports'), ' '.repeat(Math.max(maxLengthOfExportName - 'Exports'.length, 0)), pc.underline('File'), ' '.repeat(Math.max(maxFilenameLength - 'File'.length, 0)), pc.underline('Size'));
    statsArray.forEach(([exportName, filesList])=>{
        // sort by file type, first js files then types, js/mjs/cjs are prioritized than .d.ts/.d.mts/.d.cts
        filesList.sort(([a], [b])=>{
            const aIsType = isTypeFile(a);
            const bIsType = isTypeFile(b);
            if (aIsType && bIsType) {
                return 0;
            }
            if (aIsType) {
                return 1;
            }
            if (bIsType) {
                return -1;
            }
            return 0;
        }).forEach((item, index)=>{
            const [filename, , size] = item;
            const normalizedExportName = normalizeExportName(exportName);
            const prefix = index === 0 ? normalizedExportName + ' '.repeat(maxLengthOfExportName - normalizedExportName.length) : ' '.repeat(maxLengthOfExportName);
            const sizePadding = ' '.repeat(maxFilenameLength - filename.length);
            const prettiedSize = prettyBytes__default.default(size);
            const isType = isTypeFile(filename);
            console.log(` ${prefix} ${pc[isType ? 'dim' : 'bold'](filename)}${sizePadding}  ${prettiedSize}`);
        });
    });
}

// TODO: support nested wildcard exportsCondition (e.g. './foo/*')
const getWildcardExports = (exportsCondition)=>{
    return {
        './*': exportsCondition['./*']
    };
};
const isExportable = async (dirent, pathname)=>{
    if (dirent.isDirectory()) {
        const innerDirents = await fs__default.default.readdir(path__default.default.join(pathname, dirent.name), {
            withFileTypes: true
        });
        return innerDirents.some(({ name })=>name.startsWith('index') && hasAvailableExtension(name));
    }
    return dirent.isFile() && !dirent.name.startsWith('index') && hasAvailableExtension(dirent.name);
};
async function getExportables(cwd, excludeKeys) {
    const pathname = path__default.default.resolve(cwd, SRC);
    const dirents = await fs__default.default.readdir(pathname, {
        withFileTypes: true
    });
    const exportables = await Promise.all(dirents.map(async (dirent)=>await isExportable(dirent, pathname) && !excludeKeys.includes(dirent.name) ? dirent.name : undefined));
    return exportables.filter(nonNullable);
}
function mapWildcard(wildcardExports, exportables) {
    return exportables.map((exportable)=>{
        const isFile = exportable.includes('.');
        const filename = isFile ? filePathWithoutExtension(exportable) : exportable;
        return {
            [`./${filename}`]: Object.fromEntries(Object.entries(wildcardExports['./*']).map(([key, value])=>[
                    key,
                    value.replace(/\*/g, isFile ? filename : `${filename}/index`)
                ]))
        };
    });
}
async function resolveWildcardExports(exportsCondition, cwd) {
    if (!exportsCondition || typeof exportsCondition === 'string') return undefined;
    const hasWildcard = !!exportsCondition['./*'];
    if (hasWildcard) {
        logger.warn(`The wildcard export "./*" is experimental and may change or be removed at any time.\n` + 'To open an issue, please visit https://github.com/huozhi/bunchee/issues' + '.\n');
        // './foo' -> ['foo']; './foo/bar' -> ['bar']
        // will contain '*' also but it's not a problem
        const excludeKeys = Object.keys(exportsCondition).map((key)=>key.split('/').pop());
        const exportables = await getExportables(cwd, excludeKeys);
        if (exportables.length > 0) {
            const wildcardExports = getWildcardExports(exportsCondition);
            const resolvedWildcardExports = mapWildcard(wildcardExports, exportables);
            const resolvedExports = Object.assign({}, exportsCondition, ...resolvedWildcardExports);
            delete resolvedExports['./*'];
            return resolvedExports;
        }
    }
    return undefined;
}

function assignDefault(options, name, defaultValue) {
    if (!(name in options) || options[name] == null) {
        options[name] = defaultValue;
    }
}
function hasMultiEntryExport(exportPaths) {
    const exportKeys = Object.keys(exportPaths).filter((key)=>key !== './package.json');
    return exportKeys.length > 0 && exportKeys.every((name)=>name.startsWith('.'));
}
async function bundle(entryPath, { cwd: _cwd, ...options } = {}) {
    const cwd = path.resolve(process.cwd(), _cwd || '');
    assignDefault(options, 'format', 'es');
    assignDefault(options, 'minify', false);
    assignDefault(options, 'target', 'es2015');
    const pkg = await getPackageMeta(cwd);
    const resolvedWildcardExports = await resolveWildcardExports(pkg.exports, cwd);
    const packageType = getPackageType(pkg);
    const exportPaths = getExportPaths(pkg, packageType, resolvedWildcardExports);
    const exportKeys = Object.keys(exportPaths).filter((key)=>key !== './package.json');
    // const exportPathsLength = Object.keys(exportPaths).length
    const isMultiEntries = hasMultiEntryExport(exportPaths) // exportPathsLength > 1
    ;
    const hasBin = Boolean(pkg.bin);
    const tsConfig = await resolveTsConfig(cwd);
    const hasTsConfig = Boolean(tsConfig == null ? void 0 : tsConfig.tsConfigPath);
    const defaultTsOptions = {
        tsConfigPath: tsConfig == null ? void 0 : tsConfig.tsConfigPath,
        tsCompilerOptions: (tsConfig == null ? void 0 : tsConfig.tsCompilerOptions) || {}
    };
    // Handle single entry file
    if (!isMultiEntries) {
        // Use specified string file path if possible, then fallback to the default behavior entry picking logic
        // e.g. "exports": "./dist/index.js" -> use "./index.<ext>" as entry
        entryPath = entryPath || await getSourcePathFromExportPath(cwd, '.', 'default') || '';
    }
    if (entryPath) {
        let mainEntryPath;
        let typesEntryPath;
        // with -o option
        if (options.file) {
            mainEntryPath = options.file;
        } else if (exportKeys.length === 0) {
            mainEntryPath = path.resolve(cwd, 'dist/index.js');
        }
        if (mainEntryPath) {
            if (options.dts) {
                typesEntryPath = getTypeFilePath(mainEntryPath, undefined, cwd);
            }
            exportPaths['.'] = constructDefaultExportCondition({
                main: mainEntryPath,
                types: typesEntryPath
            }, packageType);
        }
    }
    const bundleOrWatch = (rollupConfig)=>{
        const { input, exportName } = rollupConfig;
        const exportPath = getExportPath(pkg, cwd, exportName);
        // Log original entry file relative path
        typeof input.input === 'string' ? path.relative(cwd, input.input) : exportPath;
        if (options.watch) {
            return Promise.resolve(runWatch(rollupConfig));
        }
        return runBundle(rollupConfig);
    };
    const hasSpecifiedEntryFile = entryPath ? await fileExists(entryPath) && (await fs__default.default.stat(entryPath)).isFile() : false;
    const hasNoEntry = !hasSpecifiedEntryFile && !isMultiEntries && !hasBin;
    if (hasNoEntry) {
        const err = new Error(`Entry file \`${entryPath}\` is not existed`);
        err.name = 'NOT_EXISTED';
        return Promise.reject(err);
    }
    const entries = await collectEntries(pkg, entryPath, exportPaths, cwd);
    const sizeCollector = createOutputState({
        entries
    });
    const entriesAlias = getReversedAlias(entries);
    const buildContext = {
        entries,
        pkg,
        exportPaths,
        cwd,
        tsOptions: defaultTsOptions,
        pluginContext: {
            outputState: sizeCollector,
            moduleDirectiveLayerMap: new Map(),
            entriesAlias
        }
    };
    const buildConfigs = await buildEntryConfig(options, buildContext, false);
    const assetsJobs = buildConfigs.map((rollupConfig)=>bundleOrWatch(rollupConfig));
    const typesJobs = hasTsConfig ? (await buildEntryConfig(options, buildContext, true)).map((rollupConfig)=>bundleOrWatch(rollupConfig)) : [];
    const result = await Promise.all(assetsJobs.concat(typesJobs));
    if (result.length === 0) {
        logger.warn('The "src" directory does not contain any entry files. ' + 'For proper usage, please refer to the following link: ' + 'https://github.com/huozhi/bunchee#usage');
    }
    logOutputState(sizeCollector);
    return result;
}
function runWatch({ input, output }, metadata) {
    const watchOptions = [
        {
            ...input,
            output: output,
            watch: {
                exclude: [
                    'node_modules/**'
                ]
            }
        }
    ];
    const watcher = rollup.watch(watchOptions);
    watcher.on('event', (event)=>{
        switch(event.code){
            case 'ERROR':
                {
                    return onError(event.error);
                }
            case 'START':
                {
                    break;
                }
            case 'END':
                {
                    break;
                }
            default:
                return;
        }
    });
    return watcher;
}
function runBundle({ input, output }) {
    return rollup.rollup(input).then((bundle)=>{
        const writeJobs = output.map((options)=>bundle.write(options));
        return Promise.all(writeJobs);
    }, onError);
}
function onError(error) {
    if (!error) return;
    // logging source code in format
    if (error.frame) {
        process.stderr.write(error.frame + '\n');
    }
    // filter out the rollup plugin error information such as loc/frame/code...
    const err = new Error(error.message);
    err.stack = error.stack;
    throw err;
}

exports.bundle = bundle;
